local _L = nUI_InfoPanel_Professions_L

local SKILLCAP = 600
local y = -5
local imgFolder = "Interface\\AddOns\\nUI_InfoPanel_Professions\\IMAGES\\"
local ProfessionsFrameBG = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background.blp", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 40, edgeSize = 20, insets = {left = 5, right = 5, top = 5, bottom = 5}}
local ProfessionsBarBG = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 40, edgeSize = 10, insets = {left = 2, right = 2, top = 2, bottom = 2}}

--  Tables
local skills = {}
local bars = {}

function Professions()
	ProfessionsFrame()
end

function ProfessionsFrame()
	-- Create Frame
	ProfessionsFrame = CreateFrame("Frame","Professions_Frame",UIParent)
	ProfessionsFrame:SetFrameStrata("LOW")
	ProfessionsFrame:SetWidth(400)
	ProfessionsFrame:SetHeight(100)
	ProfessionsFrame:SetBackdrop(ProfessionsFrameBG)

	ProfessionsFrame:SetClampedToScreen(true)
	ProfessionsFrame:EnableMouse(true)

	-- Event Handlers
	ProfessionsFrame:RegisterEvent("SKILL_LINES_CHANGED")
	ProfessionsFrame:RegisterEvent("ADDON_LOADED")
	ProfessionsFrame:SetScript("OnEvent", ProfessionsOnEvent)

	ProfessionsFrame:Show()
end

function ProfessionsDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
	local bar = bars[name]
	if not bar then --if bar doesn't exist then make the Skill bar
		bar = CreateFrame("StatusBar", nil, ProfessionsFrame,  UIParent)
		bar:SetFrameStrata("LOW")
		bar:SetPoint("TOPRIGHT", "ProfessionsFrame", "TOPRIGHT", -5, y)
		bar:SetPoint("TOPLEFT", "ProfessionsFrame", "TOPLEFT", 5, y)
		bar:SetSize(290,20)
		bar:SetBackdrop(ProfessionsBarBG)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetStatusBarTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar")
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetMinMaxValues(0,1)
		bar:SetValue((rank + rankModifier)/maxRank)

		local barTxt = bar:CreateFontString(nil, "ARTWORK")
		isValid = barTxt:SetFont("Fonts\\FRIZQT__.TTF", 16)
		barTxt:SetPoint("CENTER", bar, "CENTER", 0, 1)
		bar.text = barTxt

		-- The first button on the skills action bar
		local barBtn1 = CreateFrame("Button", nil, bar, "SecureActionButtonTemplate")
		barBtn1:SetFrameStrata("LOW")
		barBtn1:SetPoint("TOPLEFT", bar, "TOPLEFT", 0, 0)
		barBtn1:SetSize(20,20)
		barBtn1:EnableMouse(true)
		barBtn1:SetAttribute("type", "spell");
		barBtn1:SetAttribute("spell", ProfessionsGetSpell(name, 1));
		barBtn1BG = { bgFile = imgFolder .. ProfessionsGetButton(name, 1) .. ".BLP", edgeFile = nil, tile = false, tileSize = 20, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
		barBtn1:SetBackdrop(barBtn1BG)
		bar.btn1 = barBtn1

		-- The second button if needed
		if numSpells == 2 then
			if name ~= "Skinning" and name ~= "Tailoring" and name ~= "Alchemy" and name ~= "Engineering" and name ~= "Blacksmithing" and name ~= "Leatherworking" and name ~= "Mining" then
				local barBtn2 = CreateFrame("Button", nil, barBtn1, "SecureActionButtonTemplate")
				barBtn2:SetFrameStrata("LOW")
				barBtn2:SetPoint("TOP", barBtn1, "TOPRIGHT", 10, 0)
				barBtn2:SetSize(20,20)
				barBtn2:EnableMouse(true)
				barBtn2:SetAttribute("type", "spell");
				barBtn2:SetAttribute("spell", ProfessionsGetSpell(name, 2));
				barBtn2BG = { bgFile = imgFolder .. ProfessionsGetButton(name, 2) .. ".BLP", edgeFile = nil, tile = false, tileSize = 20, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
				barBtn2:SetBackdrop(barBtn2BG)
				bar.btn2 = barBtn2
			end
		end

		-- Text formatting for the text on the Skill Bar
		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)

		-- Add to the array
		bars[name] = bar
	else 
		bar:ClearAllPoints()
	end

	-- if the rank is in trainable range and not skill cap then turn bar red
	if	rank >= maxRank - 25 and maxRank < SKILLCAP then
		bar:SetStatusBarColor(0.7, 0.0, 0.0, 1.0)
		bar:SetBackdropColor(0.7, 0.0, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFF0000%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	else -- otherwise keep bar green		
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	end

	ProfessionsFrame:SetHeight((y - 20)* -1)
end

function ProfessionsUpdateData()
	-- Capture the SkillIndes for the players Professions
	local prof1, prof2, archaeology, cooking, fishing, firstAid, lockpicking = GetProfessions()

	-- Build an array of the Professions
	local profs = {prof1, prof2, archaeology, cooking, fishing, firstAid, lockpicking}

	for k, v in pairs(profs) do
		-- Fetch the details for each Profession
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier = GetProfessionInfo(v)
		ProfessionsDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
		y = y - 25

		table.insert( skills, name)
	end

	ProfessionsCleanUp()

	y = -5
end

function ProfessionsCleanUp()
	local match = false
	local clnBar

	for k, d in pairs(bars) do -- cycle bar names
		for i, v in ipairs(skills) do -- cycle skill names
			if k == v then -- if the names match set true
				match = true
				break
			else -- otherwise false
				match = false
			end
		end

		-- store bar to local variable
		clnBar = bars[k]

		if match == false then -- If bar and skill don't match hide bar and set to false
			clnBar:Hide()
			match = false
		elseif match == true then -- If bar and skill match show
			clnBar:Show()
		end
	end

	while ((# skills) > 0) do -- cycle through skills
		table.remove( skills, 1 ) -- remove skills from the table
	end
end

function ProfessionsOnEvent(self, event, arg1)
	if event == "SKILL_LINES_CHANGED" then
		if not InCombatLockdown() then
			ProfessionsUpdateData()
		end
	elseif event == "ADDON_LOADED" then
		if ProfessionsAutohide == nil then
			ProfessionsAutohide = false
		end
	end
end

function ProfessionsGetButton(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Campfire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "FirstAid"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospect"
		end
	elseif name == "LockPicking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return "Smelt"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

function ProfessionsGetSpell(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Cooking Fire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "First Aid"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return "Herb Gathering"
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospecting"
		end
	elseif name == "LockPicking" then
		if spellNum == 1 then
			return "LockPicking"
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return "Smelting"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "MoA"
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

Professions()
